@@ -581,19 +581,19 @@ attribute_is_sensitive (const CK_ATTRIBUTE *attr)
 	X (CKA_X_PEER)
 	X (CKA_X_DISTRUSTED)
 	X (CKA_X_CRITICAL)
-	X (CKA_NETSCAPE_URL)
-	X (CKA_NETSCAPE_EMAIL)
-	X (CKA_NETSCAPE_SMIME_INFO)
-	X (CKA_NETSCAPE_SMIME_TIMESTAMP)
-	X (CKA_NETSCAPE_PKCS8_SALT)
-	X (CKA_NETSCAPE_PASSWORD_CHECK)
-	X (CKA_NETSCAPE_EXPIRES)
-	X (CKA_NETSCAPE_KRL)
-	X (CKA_NETSCAPE_PQG_COUNTER)
-	X (CKA_NETSCAPE_PQG_SEED)
-	X (CKA_NETSCAPE_PQG_H)
-	X (CKA_NETSCAPE_PQG_SEED_BITS)
-	X (CKA_NETSCAPE_MODULE_SPEC)
+	X (CKA_NSS_URL)
+	X (CKA_NSS_EMAIL)
+	X (CKA_NSS_SMIME_INFO)
+	X (CKA_NSS_SMIME_TIMESTAMP)
+	X (CKA_NSS_PKCS8_SALT)
+	X (CKA_NSS_PASSWORD_CHECK)
+	X (CKA_NSS_EXPIRES)
+	X (CKA_NSS_KRL)
+	X (CKA_NSS_PQG_COUNTER)
+	X (CKA_NSS_PQG_SEED)
+	X (CKA_NSS_PQG_H)
+	X (CKA_NSS_PQG_SEED_BITS)
+	X (CKA_NSS_MODULE_SPEC)
 	X (CKA_TRUST_DIGITAL_SIGNATURE)
 	X (CKA_TRUST_NON_REPUDIATION)
 	X (CKA_TRUST_KEY_ENCIPHERMENT)
@@ -636,12 +636,12 @@ format_class (p11_buffer *buffer,
 	X (CKO_MECHANISM)
 	X (CKO_X_TRUST_ASSERTION)
 	X (CKO_X_CERTIFICATE_EXTENSION)
-	X (CKO_NETSCAPE_CRL)
-	X (CKO_NETSCAPE_SMIME)
-	X (CKO_NETSCAPE_TRUST)
-	X (CKO_NETSCAPE_BUILTIN_ROOT_LIST)
-	X (CKO_NETSCAPE_NEWSLOT)
-	X (CKO_NETSCAPE_DELSLOT)
+	X (CKO_NSS_CRL)
+	X (CKO_NSS_SMIME)
+	X (CKO_NSS_TRUST)
+	X (CKO_NSS_BUILTIN_ROOT_LIST)
+	X (CKO_NSS_NEWSLOT)
+	X (CKO_NSS_DELSLOT)
 	#undef X
 	}
 
@@ -704,7 +704,7 @@ format_key_type (p11_buffer *buffer,
 	X (CKK_AES)
 	X (CKK_BLOWFISH)
 	X (CKK_TWOFISH)
-	X (CKK_NETSCAPE_PKCS8)
+	X (CKK_NSS_PKCS8)
 	#undef X
 	}
 
@@ -741,11 +741,11 @@ format_trust_value (p11_buffer *buffer,
 
 	switch (trust) {
 	#define X(x) case x: string = #x; break;
-	X (CKT_NETSCAPE_TRUSTED)
-	X (CKT_NETSCAPE_TRUSTED_DELEGATOR)
-	X (CKT_NETSCAPE_UNTRUSTED)
-	X (CKT_NETSCAPE_MUST_VERIFY)
-	X (CKT_NETSCAPE_TRUST_UNKNOWN)
+	X (CKT_NSS_TRUSTED)
+	X (CKT_NSS_TRUSTED_DELEGATOR)
+	X (CKT_NSS_NOT_TRUSTED)
+	X (CKT_NSS_MUST_VERIFY_TRUST)
+	X (CKT_NSS_TRUST_UNKNOWN)
 	}
 
 	if (string != NULL)
@@ -838,7 +838,7 @@ format_attribute_type (p11_buffer *buffer,
 	X (CKA_BASE)
 	X (CKA_PRIME_BITS)
 	/* X (CKA_SUBPRIME_BITS) */
-	/* X (CKA_SUB_PRIME_BITS) */
+	X (CKA_SUB_PRIME_BITS)
 	X (CKA_VALUE_BITS)
 	X (CKA_VALUE_LEN)
 	X (CKA_EXTRACTABLE)
@@ -880,19 +880,19 @@ format_attribute_type (p11_buffer *buffer,
 	X (CKA_X_PEER)
 	X (CKA_X_DISTRUSTED)
 	X (CKA_X_CRITICAL)
-	X (CKA_NETSCAPE_URL)
-	X (CKA_NETSCAPE_EMAIL)
-	X (CKA_NETSCAPE_SMIME_INFO)
-	X (CKA_NETSCAPE_SMIME_TIMESTAMP)
-	X (CKA_NETSCAPE_PKCS8_SALT)
-	X (CKA_NETSCAPE_PASSWORD_CHECK)
-	X (CKA_NETSCAPE_EXPIRES)
-	X (CKA_NETSCAPE_KRL)
-	X (CKA_NETSCAPE_PQG_COUNTER)
-	X (CKA_NETSCAPE_PQG_SEED)
-	X (CKA_NETSCAPE_PQG_H)
-	X (CKA_NETSCAPE_PQG_SEED_BITS)
-	X (CKA_NETSCAPE_MODULE_SPEC)
+	X (CKA_NSS_URL)
+	X (CKA_NSS_EMAIL)
+	X (CKA_NSS_SMIME_INFO)
+	X (CKA_NSS_SMIME_TIMESTAMP)
+	X (CKA_NSS_PKCS8_SALT)
+	X (CKA_NSS_PASSWORD_CHECK)
+	X (CKA_NSS_EXPIRES)
+	X (CKA_NSS_KRL)
+	X (CKA_NSS_PQG_COUNTER)
+	X (CKA_NSS_PQG_SEED)
+	X (CKA_NSS_PQG_H)
+	X (CKA_NSS_PQG_SEED_BITS)
+	X (CKA_NSS_MODULE_SPEC)
 	X (CKA_TRUST_DIGITAL_SIGNATURE)
 	X (CKA_TRUST_NON_REPUDIATION)
 	X (CKA_TRUST_KEY_ENCIPHERMENT)
