@@ -64,10 +64,32 @@
 #endif
 #endif
 
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
 #ifndef HAVE_GETPROGNAME
 const char * getprogname (void);
 #endif
 
+#ifndef HAVE_BASENAME
+
+char *       basename    (const char *name);
+
+#endif /* HAVE_BASENAME */
+
+#ifndef HAVE_MKSTEMP
+
+int          mkstemp     (char *template);
+
+#endif /* HAVE_MKSTEMP */
+
+#ifndef HAVE_MKDTEMP
+
+char *       mkdtemp     (char *template);
+
+#endif /* HAVE_MKDTEMP */
+
 /* -----------------------------------------------------------------------------
  * WIN32
  */
@@ -105,9 +127,9 @@ typedef DWORD p11_thread_id_t;
 
 typedef void * (*p11_thread_routine) (void *arg);
 
-int p11_thread_create (thread_t *thread, thread_routine, void *arg);
+int p11_thread_create (p11_thread_t *thread, p11_thread_routine, void *arg);
 
-int p11_thread_join (thread_t thread);
+int p11_thread_join (p11_thread_t thread);
 
 /* Returns a thread_id_t */
 #define p11_thread_id_self() \
@@ -115,18 +137,26 @@ int p11_thread_join (thread_t thread);
 
 typedef HMODULE dl_module_t;
 
-#define p11_module_open(f) \
+#define p11_dl_open(f) \
 	(LoadLibrary (f))
-#define p11_module_close(d) \
+#define p11_dl_close(d) \
 	(FreeLibrary (d))
-#define p11_module_symbol(d, s) \
+#define p11_dl_symbol(d, s) \
 	((void *)GetProcAddress ((d), (s)))
 
-const char *    p11_module_error       (void);
+char *    p11_dl_error       (void);
 
 #define p11_sleep_ms(ms) \
 	(Sleep (ms))
 
+typedef struct _p11_mmap p11_mmap;
+
+p11_mmap *  p11_mmap_open   (const char *path,
+                             void **data,
+                             size_t *size);
+
+void        p11_mmap_close  (p11_mmap *map);
+
 #endif /* OS_WIN32 */
 
 /* ----------------------------------------------------------------------------
@@ -165,21 +195,29 @@ typedef void * (*p11_thread_routine) (void *arg);
 
 typedef void * dl_module_t;
 
-#define p11_module_open(f) \
+#define p11_dl_open(f) \
 	(dlopen ((f), RTLD_LOCAL | RTLD_NOW))
-#define p11_module_close(d) \
-	(dlclose(d))
-#define p11_module_error() \
-	(dlerror ())
-#define p11_module_symbol(d, s) \
+#define p11_dl_close \
+	dlclose
+#define p11_dl_symbol(d, s) \
 	(dlsym ((d), (s)))
 
+char * p11_dl_error (void);
+
 #define p11_sleep_ms(ms) \
 	do { int _ms = (ms); \
 	struct timespec _ts = { _ms / 1000, (_ms % 1000) * 1000 * 1000 }; \
 	nanosleep (&_ts, NULL); \
 	} while(0)
 
+typedef struct _p11_mmap p11_mmap;
+
+p11_mmap *  p11_mmap_open   (const char *path,
+                             void **data,
+                             size_t *size);
+
+void        p11_mmap_close  (p11_mmap *map);
+
 #endif /* OS_UNIX */
 
 #ifdef	HAVE_ERRNO_H
@@ -201,6 +239,13 @@ void *     memdup           (const void *data,
 
 #endif /* HAVE_MEMDUP */
 
+#ifndef HAVE_STRNDUP
+
+char *     strndup          (const char *data,
+                             size_t length);
+
+#endif /* HAVE_STRDUP */
+
 #ifdef HAVE_STDBOOL_H
 #include <stdbool.h>
 #else
@@ -214,4 +259,36 @@ char *     strconcat        (const char *first,
 
 #endif /* HAVE_STRCONCAT */
 
+#ifndef HAVE_ASPRINTF
+
+int        asprintf         (char **strp,
+                             const char *fmt,
+                             ...);
+
+#endif /* HAVE_ASPRINTF */
+
+#ifndef HAVE_VASPRINTF
+#include <stdarg.h>
+
+int        vasprintf        (char **strp,
+                             const char *fmt,
+                             va_list ap);
+
+#endif /* HAVE_VASPRINTF */
+
+#ifndef HAVE_GMTIME_R
+#include <time.h>
+
+struct tm * gmtime_r        (const time_t *timep,
+                             struct tm *result);
+
+#endif /* HAVE_GMTIME_R */
+
+#ifndef HAVE_TIMEGM
+#include <time.h>
+
+time_t      timegm          (struct tm *tm);
+
+#endif /* HAVE_TIMEGM */
+
 #endif /* __COMPAT_H__ */
