@@ -421,7 +421,7 @@ main (void)
 	CuSuite* suite = CuSuiteNew ();
 	int ret;
 
-	setenv ("P11_KIT_STRICT", "1", 1);
+	putenv ("P11_KIT_STRICT=1");
 	p11_debug_init ();
 
 	SUITE_ADD_TEST (suite, test_parse_extended_key_usage);
