@@ -59,7 +59,7 @@ typedef struct _Callback {
  *
  * Used to iterate over PKCS\#11 objects.
  */
-typedef struct p11_kit_iter {
+struct p11_kit_iter {
 
 	/* Iterator matching data */
 	CK_INFO match_module;
@@ -93,7 +93,7 @@ typedef struct p11_kit_iter {
 	int iterating : 1;
 	int match_nothing : 1;
 	int keep_session : 1;
-} P11KitIter;
+};
 
 /**
  * p11_kit_iter_new:
@@ -482,10 +482,6 @@ move_next_session (P11KitIter *iter)
 		if (rv != CKR_OK || !p11_match_uri_token_info (&iter->match_token, &tinfo))
 			continue;
 
-		/* Token is not initialized, we're not going to get further, so skip */
-		if (!(tinfo.flags & CKF_TOKEN_INITIALIZED))
-			continue;
-
 		rv = (iter->module->C_OpenSession) (iter->slot, iter->session_flags,
 		                                    NULL, NULL, &iter->session);
 		if (rv != CKR_OK)
