@@ -88,7 +88,8 @@ main (void)
 	CuSuite* suite = CuSuiteNew ();
 	int ret;
 
-	setenv ("P11_KIT_STRICT", "1", 1);
+	putenv ("P11_KIT_STRICT=1");
+	p11_library_init ();
 	p11_debug_init ();
 	p11_message_quiet ();
 
